/**
 * @file mathlib_extra.h
 * @brief Special, additional math algorithms for floating-point values
 *
 * These are special, custom functions for influencing floating-point variables,
 * Usually with values that start and end either between 0.f and 1.f or
 * -1.f and +1.f.  They can push variables toward or away from the
 * minimum and maxumium possible values in a non-linear, curved shape.
 * Unless stated otherwise, "x" is the original value to be altered, and
 * "y" is a value between 0.f and 1.f that affects how strong the curve is,
 * with 1.f having the most effect.
 *
 * NOTE: A special .PNG file will also be available that will show
 * these functions on a graph, to help in choosing which one is most
 * suitable to use.  (Please see "mathlib_extra.png")
 */

/*
Copyright (C) 2002-2011 UFO: Alien Invasion.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef MATHLIB_EXTRA_H
#define MATHLIB_EXTRA_H

#include "ufotypes.h"
#include "mathlib.h"

/* matches value in gcc v2 math.h */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

/**
 * This "DENORM" is for avoiding denormal-related issues when values equal to a
 * perfect 0.0f are passed to some functions, which could otherwise
 * cause very high CPU use (slow things down), or sometimes even
 * break stuff (error, crash, etc.)
 */
#ifndef DENORM
#define DENORM 0.00000000000000000001
#endif
#ifndef ChkDn(x)
#define ChkDn(x) ((x == 0.0) ? (x = DENORM))
#endif

/** x = 0.0f to 1.0f for these functions */

/* Basic curves */
#define FpCurveUp(x,y) ((((1.0-y)+1.0)*x) / ((1.0-y)+x))
#define FpCurveDn(x,y) (1.0 - ((((1.0-y)+1.0)*(1.0-x)) / ((1.0-y)+(1.0-x))))

/* "RS" = "Rapid Start", curve has a different, skewed shape */
#define FpCurveUpRS(x,y) ((((((1.0-y)+1.0)*x) / ((1.0-y)+x)) + x) * 0.5)
#define FpCurveDnRS(x,y) (1.0-((((((1.0-y)+1.0)*(1.0-x)) / ((1.0-y)+(1.0-x))) + (1.0-x)) * 0.5))

/** These next functions can curve up OR down, depending on the
 * starting value of "x" and the center mid-point, which
 * by default is 0.5f.  The "_u_" in the function name
 * means "unsigned" because all the values are still between
 * 0.0f and 1.0f, while later functions with "_s_" will work
 * with values between -1.0f and +1.0f.
 */
/* "z" = center point, function curves "x" toward "z" */
double FpCurve1D_u_in(double x, double y, double z = 0.5f);
/* "z" = center point, function curves "x" away from "z" */
double FpCurve1D_u_out(double x, double y, double z = 0.5f);

/**
 * @brief <?? I don't understand this:  x = -1.0f to +1.0f beyond here>
 */
#define FpCurveOut(x, y) (((2.0 - (y)) * (x)) / (1.0 - (y) + fabs(x)))

#endif
